      PROGRAM CTFFIT
C     **********************************************************************
C     *   1. read images from one image file and do 2D FFT                 *
C     *   2. average these 2D FFTs                                         *
C     *   3. calculate some stats                                          * 
C     *   4. compute the angle using CC the uphalf and low half            *
C     *   5. fit the background and substract the backgourd from avg 2D DFT*
C     *   6. fit the defocus and tempfactor                                *
C     *   7. write out                                                     *
C     **********************************************************************
      implicit none

      INCLUDE '../include/allprogs.inc'
      INCLUDE '../include/pifhead.inc'

      type (PIFHEAD_T) MAPHEAD, IMGHEAD
      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD
      
      Character(len=8) ::   subname = '    '

C     Local variables, used only in the main routine
      integer wind, alloc_stat
      parameter (wind=5)
      real center(2)
      real, allocatable ::  pixels(:), rffta_omega(:), ctf2d(:), 
     +     fasum(:), bakfasum(:)
      complex, allocatable :: fft2d(:)
      double precision, allocatable :: fftasum(:), fftasumsq(:), rotd(:)
      double precision amp, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD

      character(len=filename_len) :: infile, outfile
      
      integer npix_col, npix_row, num_img, nrffta_row,nrffta_col,
     +     nrffta_omega, nfc_2d_row, nfc_2d_col, hkl_max,
     +     i, j , mem_size, array_size
      integer astart, aend, i_omega, k, k1, k2, kk, icount
      integer itemp, itemp2
      real    pixel_size, volt, tempfac, wavelength,
     +     a, Cs, defmajor, defminor, angle
      real    rtemp, rtemp1, rtemp2, tmaj, tmin, fw2, ttemp
      real    omega, annual, l_ann, astep, maxcc, ccvalue, mincc
      double precision minvalue, sumvalue, a1, a2, 
     +     a1sum, a2sum, a1sqsum, a2sqsum, a1a2sum
      real   cc(-wind:wind,-wind:wind,-wind:wind)
      integer   dfocus(3)
      
      double precision t1, t2, wtime, time, t_init, t_interp,
     +     mem_total, mem_pixels_array, mem_peak
      
      logical test_time, test_memory
      
C     *********************************************************************
C     *                      Initialization                               *
C     *********************************************************************
      PROGNAME = 'CTFFIT'
      test_time   = .true.
      test_memory = .true.
      call usage
C     
      t1 = wtime()
C     
C     --------------------------------------------------------------------
C     read input image file name and out put file name
 100  print *, 'Enter Input image FILENAME: '
      READ (*,'(A)',ERR=100) infile
      print *, 'Input image FILENAME: ', infile
C     
      IMGHEAD%LUNUM = pix_file_unit
      CALL PIF_OPEN (infile, IMGHEAD%LUNUM, 'r')
      CALL PIF_READ_GH (IMGHEAD)
      npix_col  = IMGHEAD%NUMCOLS
      npix_row  = IMGHEAD%NUMROWS
      num_img   = IMGHEAD%nimg
C     
 101  print *, 'Enter Output fited FILENAME: '
      READ (*,'(A)',ERR=101) outfile
      print *, 'Output fited FILENAME: ', outfile
      
C     --------------------------------------------------------------------
C     read and determine the fft size
 300  print *, 'Enter FFT size:'
      READ (*,*,ERR=300) itemp
      if (itemp .LT. npix_col) goto 300
      call TRANS_LENGTH_ESSL( itemp, nrffta_row )
      print *, ' FFT size =',nrffta_row
C     
      nfc_2d_row   = nrffta_row/2
      nfc_2d_col   = nfc_2d_row
      hkl_max      = nfc_2d_row
      nrffta_col   = nrffta_row
      nrffta_omega = nrffta_col + 15
C     --------------------------------------------------------------------
C     read pixel size 
 301  print *, 'Enter pixel_size:'
      READ (*,*,ERR=301) rtemp
      if (rtemp .LE. 0.0) goto 301
      pixel_size = rtemp
      print *, ' pixel_size =', pixel_size
C     --------------------------------------------------------------------
C     read voltage and tempfac
 302  print *, 'Enter voltage and temperature factor:'
      READ (*,*,ERR=302) rtemp, rtemp1
      if ((rtemp .Lt. 0.0) .or. (rtemp1 .Lt. 0.0)) goto 302
      volt    = rtemp
      tempfac = rtemp1
      print *, ' voltage =', volt, 'temperature =', tempfac
C     --------------------------------------------------------------------
C     read guess defocus value
 303  print *, 'Enter guessed defocus values and angle:'
      READ (*,*,ERR=303) rtemp, rtemp1, rtemp2
      if ((rtemp .LE. 0.0) .or. (rtemp1 .le. 0.0)) goto 303
      defmajor = rtemp
      defminor = rtemp1
      angle    = rtemp2
      print *,'guessed defocus values and angle =',
     +     defmajor, defminor, angle
C     --------------------------------------------------------------------
C     Percentage of Amplitude Contrast & Spherical aberration constant
      A = 0.05
      Cs = 2.0
 314  print *,'Enter Amplitude Contrast & Spherical aberration constant'
      READ (*,*,ERR=314) rtemp, rtemp1
      if ((rtemp .le. 0.) .or. (rtemp1 .le. 0 )) goto 314
      A   = rtemp
      Cs  = rtemp1
      print *, 'Percentage Amp Contrast =', rtemp, '  Cs =', rtemp1
      
C     alloc memory
      array_size   = (nfc_2d_row +1) * (nfc_2d_col * 2 +1)
      allocate(rffta_omega(nrffta_omega), pixels(npix_col*npix_row),
     +     fftasum(array_size), fftasumsq(array_size),
     +     fft2d(array_size), rotd(array_size),
     +     stat = alloc_stat)
      if(alloc_stat .gt. 0) then
         call error_stop(progname, subname,
     +        'Error allocating memory for image and FFTs')
      endif

      mem_pixels_array = npix_col * npix_row * 4
      mem_size = 4*nrffta_omega + mem_pixels_array +
     +     4*8*array_size
      mem_peak = mem_size

      do i = 1, array_size
         fftasum(i)   = 0.
         fftasumsq(i) = 0.
      enddo

      t2 = wtime()
      t_init = t2 - t1

C     read each image do 2D DFT to fft2d(complex) calculate fftasum and
C     fftasumsq(real)

      call vrffti( nrffta_col, rffta_omega )
      itemp  = npix_row*npix_col
      do i = 1, num_img

C     read one image
         IMGHEAD%IMGNUM = i
         CALL PIF_READ_DH(IMGHEAD)
         call rpifimag_f (itemp, pixels)

C     do 2D DFT
         center(1) = (npix_row-1)/2.0
         center(2) = (npix_col-1)/2.0
         call fft_2dfft(pixels, npix_row, npix_col, center,
     +        nrffta_omega, rffta_omega, nrffta_row, nrffta_col,
     +        hkl_max, hkl_max, fft2d)

C     average them   
         do j = 1, array_size
            amp       = sqrt( real(fft2d(j))**2 + aimag(fft2d(j))**2)
            fftasum(j) = fftasum(j) + amp
            fftasumsq(j) = fftasumsq(j) + amp**2
         enddo

      enddo

      call pif_close (pix_file_unit)
      deallocate(rffta_omega, fft2d, pixels)
      mem_total = mem_total - mem_pixels_array - 2*8*array_size

C     *********************************************************************
C     caluate some stats, we use fftasum to store avg, fftasumsq store std
C     *********************************************************************

      MAP_MIN = fftasum(1)/num_img
      MAP_MAX = fftasum(1)/num_img
      MAP_AVG = 0. 
      MAP_STD = 0.
      do i = 1, array_size
         amp = fftasumsq(i)/num_img - fftasum(i)**2
         fftasum(i) = fftasum(i) / num_img
         if ( amp .gt.0 ) then 
            fftasumsq(i) = sqrt(amp)
         else
            fftasumsq(i) = 0
         endif
         MAP_MIN = MIN(MAP_MIN, fftasum(i))
         MAP_MAX = MAX(MAP_MAX, fftasum(i))
         MAP_AVG = MAP_AVG + fftasum(i)
         MAP_STD = MAP_STD + fftasum(i)**2
      enddo

      MAP_AVG = MAP_AVG / array_size
      amp     = MAP_STD / array_size - MAP_AVG**2
      if (amp.gt.0 ) MAP_STD = sqrt(amp)

      do i = 1, array_size
         fftasum(i) = fftasum(i) / MAP_AVG
         fftasumsq(i) = fftasumsq(i) / MAP_AVG
      enddo

C     call doutimg(fftasum, -hkl_max, hkl_max,"ampfile2")

      WRITE(*,1000)
      WRITE(*,1001)
 1000 FORMAT('                  STATICS: averaged FFT')
 1001 FORMAT('FFTSIZE   MINIMUM     MAXIMUM        AVERAGE        STD',
     1     /,' -----------------------------------------------------',/)
      WRITE(*,*)hkl_max, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD

C     *********************************************************************
C     note: we have defferent angle define in FFT space. 
C     Because the start pointer of our FFT space is in upleft corner
C     while Robem is in the down left corner
C     compute angle which is +- 45 around input angle.
      annual = real(hkl_max)
      astep  = 1.0
      astart =  - 45
      aend   = 45
      do kk = 1, 3
         i_omega = astart 
         do i =  astart, aend
            omega = i * astep - angle
            call rotate(fftasum, hkl_max, hkl_max, rotd, omega, annual)
            a1sum = 0.0
            a2sum = 0.0
            a1sqsum = 0.0
            a2sqsum = 0.0
            a1a2sum = 0.0
            icount = 0
            do j = 1, hkl_max+1
               do k = 0, hkl_max-1
                  k1 = k*(hkl_max+1) + j
                  k2 = (2*hkl_max - k)*(hkl_max+1) + j
                  a1 = rotd(k1)
                  a2 = rotd(k2)
                  a1sum = a1sum + a1
                  a2sum = a2sum + a2
                  a1sqsum = a1sqsum + a1**2
                  a2sqsum = a2sqsum + a2**2
                  a1a2sum = a1a2sum + a1*a2
                  icount = icount + 1
               enddo
            enddo
            sumvalue = (a1a2sum - a1sum*a2sum / icount ) 
     +           / sqrt( a1sqsum - a1sum**2 /icount) 
     +           / sqrt( a2sqsum - a2sum**2 /icount) 
            
            if ( i.eq.astart) minvalue = sumvalue
C     print *, i, omega, sumvalue, minvalue
            if ( minvalue.lt.sumvalue) then 
               minvalue = sumvalue
               i_omega  = i
            endif
         enddo
         angle  = - i_omega * astep + angle
         print *, 'new angle=', angle, i_omega, minvalue
         astep  = astep / 10.
         astart =  - 9
         aend   = 9
      enddo

      deallocate(rotd, fftasumsq)
      mem_total = mem_total - 2*8*array_size

      allocate(ctf2d(array_size), fasum(array_size),
     +     stat = alloc_stat)
      if(alloc_stat .gt. 0) then
         call error_stop(progname, subname, 
     +        'ctf space memory allocation failed')
      endif

      mem_total = mem_total + 2*4*array_size
      mem_peak  = max(mem_peak, mem_total)

      do i = 1, array_size
         fasum(i) = fftasum(i)
      enddo
      deallocate(fftasum)
      mem_total = mem_total - 8*array_size

C     call soutimg(fasum, hkl_max, hkl_max,"ampsumavg\0")
C     
C     *********************************************************************
C     fit the background and substract
C     Iout = Iin*CTF*Tempfactor + Sn, Sn=a*exp(-dstar/C**2) is the background
C     How to get rid of the background?
C     reduce bakground and decay of the averaged FFT image

      allocate(bakfasum(array_size), stat = alloc_stat)
      if(alloc_stat .gt. 0) then
         call error_stop(progname, subname, 
     +        'bakfasum memory allocation failed')
      endif

      mem_total = mem_total + 4*array_size
      mem_peak  = max(mem_peak, mem_total)
      
      wavelength = 1.2264264967/sqrt(volt + 0.9784663543E-6 * volt**2)
      tmaj = max(defmajor,defminor)
      if (tmaj .LE. sqrt(Cs*wavelength*2.0)) call  error(
     +     progname, subname, 'defocus is too small')
      fw2 = (tmaj-sqrt(tmaj*tmaj-Cs*wavelength*2.0))/Cs
      l_ann = sqrt(fw2/1000.)*(pixel_size*2*hkl_max)/wavelength/10.
      if (l_ann .lt.10.) l_ann = 10.
      
      call imgfilter(fasum, hkl_max, hkl_max, bakfasum, l_ann)

C     call soutimg(bakfasum, hkl_max, hkl_max,"ampbfilt\0")
C     *********************************************************************
C     fit the defocus and tempfactor
C     Iout = Iin*CTF*Tempfactor + Sn, Sn=b*exp(-dstar**2 * c - dstar *
C     d) is the background
C     calculate ctf values, I use ctf2d to store ctf values.
C     using ctf_para.f, and correlation of da2d and fftasum

      annual = real(hkl_max)*3/4
      mincc  = -1.
      maxcc  = mincc
      astep  = 0.1

 1101 do k = -wind, wind
         do j = -wind, wind
            do i = -wind, wind
               cc(i,j,k) = mincc
            enddo
         enddo
      enddo

C     in order to use old formura with new unit, I change the values.
      tmaj = min(defmajor,defminor)
      if (tmaj .LE. sqrt(Cs*wavelength*2.0)) call  error(
     +     progname, subname, 'defocus is too small')
      fw2 = (tmaj-sqrt(tmaj*tmaj-Cs*wavelength*2.0))/Cs
      l_ann = sqrt(fw2/1000.)*(pixel_size*2*hkl_max)/wavelength/10.
      if (l_ann .GE. annual) call  error(
     +     progname, subname, 'defocus is too small')
      if (l_ann .lt.10.) l_ann = 10.
      print *, 'radius =',l_ann, annual

 1100 dfocus(1) = 0
      dfocus(2) = 0
      dfocus(3) = 0

      do k = -wind, wind
         ttemp = tempfac + k*astep*100
         do j = -wind, wind
            tmin = defminor + j*astep
            do i = -wind, wind
               tmaj = defmajor + i*astep
               if ((cc(i,j,k).le.mincc) .and. (tmaj.gt.0.0) .and.
     +              (tmin.gt.0.0) .and. (ttemp.ge.0.0)) then 
                  call ctf_temp(ctf2d, hkl_max, hkl_max, pixel_size,
     +                 volt, tmaj, tmin, angle, A, Cs, ttemp)
                  call corr(ctf2d, bakfasum, hkl_max, hkl_max, 
     +                 l_ann, annual, ccvalue)
                  cc(i,j,k) = ccvalue
                  if (ccvalue.GT.maxcc) then
                     dfocus(1)= i
                     dfocus(2)= j
                     dfocus(3)= k
                     maxcc= ccvalue
                  endif
               endif
            enddo
         enddo
      enddo

      defmajor = defmajor + dfocus(1) * astep
      defminor = defminor + dfocus(2) * astep
      tempfac  = tempfac  + dfocus(3) * astep * 100

      print *, 'move', dfocus(1), dfocus(2), dfocus(3), 
     +     maxcc, defmajor, defminor, tempfac

      if (abs(dfocus(1)).GE.nint(wind*0.3).or.
     +     abs(dfocus(2)).GE.nint(wind*0.3).or. 
     +     abs(dfocus(3)).GE.nint(wind*0.3) ) then
         call recenter_array_3d(cc, wind, dfocus, mincc)
         goto 1100
      endif

      if (astep.GT.0.001) then
         astep = astep/(wind+1)
         goto 1101
      endif

C     print out the result and write files
 950  OPEN (UNIT=13, FILE=outfile, ERR=960)
      goto 961

 960  call error_stop(progname, subname,
     +     'Error in opening output file')

C     open output file ok, print out the head for orientation file.

 961  WRITE(13,*) infile
      WRITE(13,1010) pixel_size, 1, volt, A,
     +     defmajor, defminor, angle, Cs, tempfac
 1010 FORMAT (1X,F8.4,',',I2,',',F10.1,',',4(F10.4,','),F8.2, F8.2)

C     write(13,*), 'avg next'
C     do i = 1, nfc_2d_row + 1
C     write(13,*), fasum((nfc_2d_row +1)*nfc_2d_col+i)
C     enddo
C     
C     write(13,*), 'filter avg next'
C     do i = 1, nfc_2d_row + 1
C     write(13,*), bakfasum((nfc_2d_row +1)*nfc_2d_col+i)
C     enddo
C     
C     write(13,*), 'temperature next'
C     call apptemp(fasum, hkl_max, hkl_max, pixel_size, tempfac)
C     do i = 1, nfc_2d_row + 1
C     write(13,*), fasum((nfc_2d_row +1)*nfc_2d_col+i)
C     enddo
C     
C     write(13,*), 'ctf next'
C     call ctf_temp(ctf2d, hkl_max, hkl_max, pixel_size, volt,
C     +              defmajor, defminor, angle, A, Cs, 0.0)
C     call soutimg(ctf2d, hkl_max, hkl_max, "ampctf\0")
C     do i = 1, nfc_2d_row + 1
C     write(13,*), ctf2d((nfc_2d_row +1)*nfc_2d_col+i)
C     enddo
C     call corr(ctf2d, fasum, hkl_max, hkl_max, 
C     +                l_ann, annual, ccvalue)
C     print *, 'CC value for apply temper =', ccvalue
C     
C     write(13,*), 'cfttemp next'
C     call ctf_temp(ctf2d, hkl_max, hkl_max, pixel_size, volt,
C     +              defmajor, defminor, angle, A, Cs, tempfac)
C     call soutimg(ctf2d, hkl_max, hkl_max, "ampctftmp\0")
C     do i = 1, nfc_2d_row + 1
C     write(13,*), ctf2d((nfc_2d_row +1)*nfc_2d_col+i)
C     enddo
C     
      close(UNIT=13)

      deallocate(fasum, bakfasum, ctf2d)
      mem_total = mem_total - 3*4*array_size
      t1 = wtime()
      t_interp = t1 - t2

      if (mem_total .ne. 0) print *,'mem_total=',mem_total,
     +     'At the end, mem_total should be 0!!!'

      time = t_init + t_interp

      if (test_time)
     +     print 40, 0, t_init,
     +     t_interp,
     +     time
 40   format (/ 'Node ', i3,
     +     ' Time used in initialization                      =',
     +     f15.4, ' ' / 8x,
     +     ' Time used in read image & 2DFFT & factors        =',
     +     f15.4, ' ' / 8x,
     +     ' Total time used                                  =',
     +     f15.4)

C     Print memory allocating message.
      if (test_memory) print 60, 0, mem_peak,
     +     mem_pixels_array, 8.*array_size, 2*4.*array_size
 60   format(/ 'Node ', i3, ' Memory allocation in BYTES:', /
     +     8x, ' Peak memory allocated   =', f18.1 /
     +     8x, ' Memory for 2D pixels    =', f18.1 /
     +     8x, ' Memory for 2D FFT       =', f18.1 /
     +     8x, ' Memory used in CMPCC    =', f18.1 )

      print *, '**********finished nicely**********'

      end program ctffit
