/* common.h 
 *
 * Common defns for robem.
 *
 * Rob Ashmore
 * Purdue University
 * Biology dept rwa@bragg.bio.purdue.edu
*/


/* Generic things */
#define FALSE 0
#define TRUE 1

/**************************
 *  TYPEDEFS              *
 **************************/
typedef char *STRING;
typedef int INTEGER;

#ifndef IS_ROBEM_MODULE 
#ifndef BYTE
typedef unsigned char BYTE;     
#endif 
#endif 

#ifdef sgi
/* SGI doens't like this */
#else
#ifdef NULL
#undef NULL
#define NULL 0
#endif
#endif

#define EQU(x,y) (((x)!=NULL &&(y)!=NULL &&(strcmp((x),(y))==0)))

#define BELL '\007'

#ifdef sgi
#define MAX(x,y)  (((x)>(y)) ? (x) : (y))  
#define MIN(x,y)  (((x)<(y)) ? (x) : (y))
#else
#ifdef VMS
#ifndef IS_MAIN_MODULE
#define MAX(x,y)  (((x)>(y)) ? (x) : (y))  
#define MIN(x,y)  (((x)<(y)) ? (x) : (y))
#endif
#else
/* OLD WAY #ifdef DECU */
/* #define MAX(x,y)  (((x)>(y)) ? (x) : (y))   */
/* #define MIN(x,y)  (((x)<(y)) ? (x) : (y)) */
#if !defined(MIN)
#define MIN(x,y)  (((x)<(y)) ? (x) : (y)) 
#endif
#if !defined(MAX)
#define MAX(x,y)  (((x)>(y)) ? (x) : (y))   
#endif
#endif
#endif


#define MAXINT 2147483647  
/* OLDWAY #define MAXSHORT 65535 */
#define MAXSHORT 32767


#define CK(x) checkMemory((int)(x), __LINE__, __FILE__)


/* cue struct */
typedef struct __cue_t__
  {/* CUE_T */
    float *depth;
    float *radial;
    float *shad;
  }/* CUE_T */ cue_t;


/* STruct for passing strings to FORTRAN routines */
typedef struct __fortran_Strings__
  {/* FORTRAN_STRINGS */
    short  length;
    BYTE   type;
    BYTE   class;
    char   *addr;
  }/* FORTRAN_STRINGS */ fString_t;


/* Struct for Purdue INT*2 file header */
typedef struct _pifHead_t
  {/* PIFHEAD_T */
    short numCols,numRows;
    float img_bkg;
    int   nimg,nrec_last;
    int   bimg_sizes;
    float img_min,img_max;
    float img_avg,img_std;
    int   file_id1, file_id2;
  }/* PIFHEAD_T */ pifHead_t;



/* Struct for Keyword input */
typedef struct _infoHead_t
  {/* INFOHEAD_T */
    int      threedr_mode;
    int      input_mode;
    int      bin_factor;
    int      symmetry;
    float    delta_theta;
    int      image_projection_compare_method;
    int      ctf_mode;
    int      ctf_filter_mode;
    int      verbose;
    float    filter_factor_1;
    float    filter_factor_2;
    float    pftrad_lo;
    float    pftrad_hi;
    float    pftrad_step;
    float    annulus_low;
    float    annulus_high;
    float    radius;
    float    temperature_factor;
    float    resolution_out;
    float    resolution_low;
    float    resolution_high;
    int      units_out;
    int      jcut;
    float    sigcut;
    float    delta_xy;
    float    num_delta_xy;
    float    delta_omega;
    float    num_delta_omega;
    float    map_pixsiz;
    float    mag_cen;
    float    mag_step;
    int      mag_num;
    int      mag_norm;
    int      map_dim;
    float    max_origin_shift;
    float    ico_spread;
    float    minimum;
    int      bg_out;
    int      lg_out;
    int      mp_out;
    char     model_filename[256];
    char     prj_filename[256];
    char     pft_filename[256];
    char     pftrads_filename[256];
    char     pftres1_filename[256];
    char     pftres2_filename[256];
    char     bg_filename[256];
    char     title[256];
    int      input_mode_cc;
    float    phase_weight;
  }/* INFOHEAD_T */ infoHead_t;



typedef struct _forPifHead_t
  {/* FORPIFHEAD_T */
    int      luNum;
    int      nimg;
    int      imgNum;
    float    floatScaleFactor;
    float    recipFloatScaleFactor;
    float    int4scaleF;
    int      htype;
    int      numCols;
    int      numRows;
    int      numSecs;
    int      mode;
    int      endianNess;
    char     versionStr[80];
    float    img_bkg;
    int      datai4_min;
    int      datai4_max;
    float    datar4_min;
    float    datar4_max;
    float    datar4_avg;
    float    datar4_std;
    int      imgOffset;
    char     descriptionStr[80];
    char     timeStr[80];
    char     micrographStr[80];
    char     scanNumberStr[80];
    float    aoverb;
    float    map_abang;
    float    a_cell;
    float    b_cell;
    float    c_cell;
    float    alp;
    float    bet;
    float    gam;
    float    dela;
    float    delb;
    float    delc;
    float    a_star;
    float    b_star;
    float    c_star;
    float    alp_star;
    float    bet_star;
    float    gam_star;
    float    pixelSize;
    int      t_matrix[6];
    int      packRadius;
    int      binFactor;
    float    dthe;
    float    dphi_90;
    int      symmetry;
    int      pixSizUnits; 
    float    res_out;
    int      ctfMode; 
    float    tempFactor;
    float    wiener;
    int      borderType;
    int      borderWidth;
    int      rad_lo;
    int      rad_hi;    
    float    pifVersion;
    int      even;    
    int      mrcX;    
    int      mrcY;    
    int      mrcZ;    
    int      transX;
    int      transY;
    int      transZ;
    int      isTrunc;
    int      origX;
    int      origY;
    int      origZ;
  }/* FORPIFHEAD_T */ forPifHead_t;



/* Struct for Structure Factors */
typedef struct _strucFact_t
  {/* STRUCFACT_T */
    short h,k,l;
    float a_amp,b_phi,sig,dstr;
  }/* STRUCFACT_T */ strucFact_t;





/* End of common.h */










